@@ -35,16 +35,9 @@ extern char *** __MINGW_IMP_SYMBOL(__initenv);
 #define __initenv (* __MINGW_IMP_SYMBOL(__initenv))
 #endif
 
-/* Hack, for bug in ld.  Will be removed soon.  */
-#if defined(__GNUC__)
-#define __ImageBase __MINGW_LSYMBOL(_image_base__)
-#endif
-/* This symbol is defined by ld.  */
 extern IMAGE_DOS_HEADER __ImageBase;
 
 extern void _fpreset (void);
-#define SPACECHAR _T(' ')
-#define DQUOTECHAR _T('\"')
 
 int *__cdecl __p__commode(void);
 
@@ -57,10 +50,10 @@ extern _CRTIMP void __cdecl _initterm(_PVFV *, _PVFV *);
 
 static int __cdecl check_managed_app (void);
 
-extern _CRTALLOC(".CRT$XIA") _PIFV __xi_a[];
-extern _CRTALLOC(".CRT$XIZ") _PIFV __xi_z[];
-extern _CRTALLOC(".CRT$XCA") _PVFV __xc_a[];
-extern _CRTALLOC(".CRT$XCZ") _PVFV __xc_z[];
+extern _PIFV __xi_a[];
+extern _PIFV __xi_z[];
+extern _PVFV __xc_a[];
+extern _PVFV __xc_z[];
 
 
 /* TLS initialization hook.  */
@@ -68,19 +61,10 @@ extern const PIMAGE_TLS_CALLBACK __dyn_tls_init_callback;
 
 extern int __mingw_app_type;
 
-HINSTANCE __mingw_winmain_hInstance;
-_TCHAR *__mingw_winmain_lpCmdLine;
-DWORD __mingw_winmain_nShowCmd = SW_SHOWDEFAULT;
-
 static int argc;
 extern void __main(void);
-#ifdef WPRFLAG
-static wchar_t **argv;
-static wchar_t **envp;
-#else
-static char **argv;
-static char **envp;
-#endif
+static _TCHAR **argv;
+static _TCHAR **envp;
 
 static int argret;
 static int mainret=0;
@@ -91,11 +75,7 @@ extern LPTOP_LEVEL_EXCEPTION_FILTER __mingw_oldexcpt_handler;
 
 extern void _pei386_runtime_relocator (void);
 long CALLBACK _gnu_exception_handler (EXCEPTION_POINTERS * exception_data);
-#ifdef WPRFLAG
-static void duplicate_ppstrings (int ac, wchar_t ***av);
-#else
-static void duplicate_ppstrings (int ac, char ***av);
-#endif
+static void duplicate_ppstrings (int ac, _TCHAR ***av);
 
 static int __cdecl pre_c_init (void);
 static void __cdecl pre_cpp_init (void);
@@ -134,7 +114,7 @@ pre_c_init (void)
   * __p__fmode() = _fmode;
   * __p__commode() = _commode;
 
-#ifdef WPRFLAG
+#ifdef _UNICODE
   _wsetargv();
 #else
   _setargv();
@@ -155,7 +135,7 @@ pre_cpp_init (void)
 {
   startinfo.newmode = _newmode;
 
-#ifdef WPRFLAG
+#ifdef _UNICODE
   argret = __wgetmainargs(&argc,&argv,&envp,_dowildcard,&startinfo);
 #else
   argret = __getmainargs(&argc,&argv,&envp,_dowildcard,&startinfo);
@@ -166,6 +146,7 @@ static int __tmainCRTStartup (void);
 
 int WinMainCRTStartup (void);
 
+__attribute__((used)) /* required due to GNU LD bug: https://sourceware.org/bugzilla/show_bug.cgi?id=30300 */
 int WinMainCRTStartup (void)
 {
   int ret = 255;
@@ -177,7 +158,11 @@ int WinMainCRTStartup (void)
 #ifdef SEH_INLINE_ASM
   asm ("\tnop\n"
     "\t.l_endw: nop\n"
+#ifdef __arm__
+    "\t.seh_handler __C_specific_handler, %except\n"
+#else
     "\t.seh_handler __C_specific_handler, @except\n"
+#endif
     "\t.seh_handlerdata\n"
     "\t.long 1\n"
     "\t.rva .l_startw, .l_endw, _gnu_exception_handler ,.l_endw\n"
@@ -192,6 +177,7 @@ int mainCRTStartup (void);
 int __mingw_init_ehandler (void);
 #endif
 
+__attribute__((used)) /* required due to GNU LD bug: https://sourceware.org/bugzilla/show_bug.cgi?id=30300 */
 int mainCRTStartup (void)
 {
   int ret = 255;
@@ -203,7 +189,11 @@ int mainCRTStartup (void)
 #ifdef SEH_INLINE_ASM
   asm ("\tnop\n"
     "\t.l_end: nop\n"
+#ifdef __arm__
+    "\t.seh_handler __C_specific_handler, %except\n"
+#else
     "\t.seh_handler __C_specific_handler, @except\n"
+#endif
     "\t.seh_handlerdata\n"
     "\t.long 1\n"
     "\t.rva .l_start, .l_end, _gnu_exception_handler ,.l_end\n"
@@ -221,14 +211,6 @@ __attribute__((force_align_arg_pointer))
 __declspec(noinline) int
 __tmainCRTStartup (void)
 {
-  _TCHAR *lpszCommandLine = NULL;
-  STARTUPINFO StartupInfo;
-  WINBOOL inDoubleQuote = FALSE;
-  memset (&StartupInfo, 0, sizeof (STARTUPINFO));
-
-  if (__mingw_app_type)
-    GetStartupInfo (&StartupInfo);
-  {
     void *lock_free = NULL;
     void *fiberid = ((PNT_TIB)NtCurrentTeb())->StackBase;
     int nested = FALSE;
@@ -275,57 +257,20 @@ __tmainCRTStartup (void)
     
     _fpreset ();
 
-    __mingw_winmain_hInstance = (HINSTANCE) &__ImageBase;
-
-#ifdef WPRFLAG
-    lpszCommandLine = (_TCHAR *) _wcmdln;
-#else
-    lpszCommandLine = (char *) _acmdln;
-#endif
-
-    if (lpszCommandLine)
-      {
-	while (*lpszCommandLine > SPACECHAR || (*lpszCommandLine && inDoubleQuote))
-	  {
-	    if (*lpszCommandLine == DQUOTECHAR)
-	      inDoubleQuote = !inDoubleQuote;
-#ifdef _MBCS
-	    if (_ismbblead (*lpszCommandLine))
-	      {
-		if (lpszCommandLine[1])
-		  ++lpszCommandLine;
-	      }
-#endif
-	    ++lpszCommandLine;
-	  }
-	while (*lpszCommandLine && (*lpszCommandLine <= SPACECHAR))
-	  lpszCommandLine++;
-
-	__mingw_winmain_lpCmdLine = lpszCommandLine;
-      }
-
-    if (__mingw_app_type)
-      {
-	__mingw_winmain_nShowCmd = StartupInfo.dwFlags & STARTF_USESHOWWINDOW ?
-				    StartupInfo.wShowWindow : SW_SHOWDEFAULT;
-      }
     duplicate_ppstrings (argc, &argv);
-    __main ();
-#ifdef WPRFLAG
+    __main (); /* C++ initialization. */
+#ifdef _UNICODE
     __winitenv = envp;
-    /* C++ initialization.
-       gcc inserts this call automatically for a function called main, but not for wmain.  */
-    mainret = wmain (argc, argv, envp);
 #else
     __initenv = envp;
-    mainret = main (argc, argv, envp);
 #endif
+    mainret = _tmain (argc, argv, envp);
     if (!managedapp)
       exit (mainret);
 
     if (has_cctor == 0)
       _cexit ();
-  }
+
   return mainret;
 }
 
@@ -370,49 +315,22 @@ check_managed_app (void)
   return 0;
 }
 
-#ifdef WPRFLAG
-static size_t wbytelen(const wchar_t *p)
+static void duplicate_ppstrings (int ac, _TCHAR ***av)
 {
-	size_t ret = 1;
-	while (*p!=0) {
-		ret++,++p;
-	}
-	return ret*2;
-}
-static void duplicate_ppstrings (int ac, wchar_t ***av)
-{
-	wchar_t **avl;
+	_TCHAR **avl;
 	int i;
-	wchar_t **n = (wchar_t **) malloc (sizeof (wchar_t *) * (ac + 1));
-
-	avl=*av;
-	for (i=0; i < ac; i++)
-	  {
-		size_t l = wbytelen (avl[i]);
-		n[i] = (wchar_t *) malloc (l);
-		memcpy (n[i], avl[i], l);
-	  }
-	n[i] = NULL;
-	*av = n;
-}
-#else
-static void duplicate_ppstrings (int ac, char ***av)
-{
-	char **avl;
-	int i;
-	char **n = (char **) malloc (sizeof (char *) * (ac + 1));
+	_TCHAR **n = (_TCHAR **) malloc (sizeof (_TCHAR *) * (ac + 1));
 	
 	avl=*av;
 	for (i=0; i < ac; i++)
 	  {
-		size_t l = strlen (avl[i]) + 1;
-		n[i] = (char *) malloc (l);
+		size_t l = sizeof (_TCHAR) * (_tcslen (avl[i]) + 1);
+		n[i] = (_TCHAR *) malloc (l);
 		memcpy (n[i], avl[i], l);
 	  }
 	n[i] = NULL;
 	*av = n;
 }
-#endif
 
 int __cdecl atexit (_PVFV func)
 {
