@@ -8,7 +8,7 @@
 
 WCHAR *gai_strerrorW(int ecode)
 {
-	DWORD dwMsgLen;
+	DWORD dwMsgLen __attribute__((unused));
 	static WCHAR buff[GAI_STRERROR_BUFFER_SIZE + 1];
 	dwMsgLen = FormatMessageW(FORMAT_MESSAGE_FROM_SYSTEM|FORMAT_MESSAGE_IGNORE_INSERTS|FORMAT_MESSAGE_MAX_WIDTH_MASK,
 				  NULL, ecode, MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT), (LPWSTR)buff,
