@@ -4,6 +4,7 @@
  * No warranty is given; refer to the file DISCLAIMER.PD within this package.
  */
 #include "fp_consts.h"
+#include <math.h>
 
 const union _ieee_rep __QNANL = { __LONG_DOUBLE_QNAN_REP };
 const union _ieee_rep __SNANL = { __LONG_DOUBLE_SNAN_REP };
@@ -15,6 +16,10 @@ const union _ieee_rep __DENORML = { __LONG_DOUBLE_DENORM_REP };
 long double nanl (const char *);
 long double nanl (const char * tagp __attribute__((unused)) )
 {
+#if defined(__arm__) || defined(_ARM_) || defined(__aarch64__) || defined(_ARM64_)
+  return nan("");
+#else
   return __QNANL.ldouble_val;
+#endif
 }
 
