@@ -42,6 +42,8 @@
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
+#define __LARGE_MBSTATE_T
+
 #include <limits.h>
 #include <stddef.h>
 #include <stdarg.h>
@@ -78,9 +80,9 @@ typedef struct _IFP
     const char *str;
   };
   int bch[1024];
-  int is_string : 1;
+  unsigned int is_string : 1;
   int back_top;
-  int seen_eof : 1;
+  unsigned int seen_eof : 1;
 } _IFP;
 
 static void *
