@@ -111,7 +111,7 @@ extern "C" {
 
 #ifndef __MIDL_user_allocate_free_DEFINED__
 #define __MIDL_user_allocate_free_DEFINED__
-  void *__RPC_USER MIDL_user_allocate(SIZE_T);
+  void *__RPC_USER MIDL_user_allocate(size_t);
   void __RPC_USER MIDL_user_free(void *);
 #endif
 
