@@ -74,6 +74,19 @@ SetThreadName_VEH (PEXCEPTION_POINTERS ExceptionInfo)
 
   return EXCEPTION_CONTINUE_SEARCH;
 }
+
+static PVOID (*AddVectoredExceptionHandlerFuncPtr) (ULONG, PVECTORED_EXCEPTION_HANDLER);
+static ULONG (*RemoveVectoredExceptionHandlerFuncPtr) (PVOID);
+
+static void __attribute__((constructor))
+ctor (void)
+{
+  HMODULE module = GetModuleHandleA("kernel32.dll");
+  if (module) {
+    AddVectoredExceptionHandlerFuncPtr = (__typeof__(AddVectoredExceptionHandlerFuncPtr)) GetProcAddress(module, "AddVectoredExceptionHandler");
+    RemoveVectoredExceptionHandlerFuncPtr = (__typeof__(RemoveVectoredExceptionHandlerFuncPtr)) GetProcAddress(module, "RemoveVectoredExceptionHandler");
+  }
+}
 #endif
 
 typedef struct _THREADNAME_INFO
@@ -355,10 +368,9 @@ pop_pthread_mem (void)
 static void
 free_pthread_mem (void)
 {
+#if 0
   _pthread_v *t;
 
-  if (1)
-  return;  
   pthread_mutex_lock (&mtx_pthr_locked);
   t = pthr_root;
   while (t != NULL)
@@ -388,6 +400,8 @@ free_pthread_mem (void)
     pthr_root = t;
   }
   pthread_mutex_unlock (&mtx_pthr_locked);
+#endif
+  return;
 }
 
 static void
@@ -398,24 +412,11 @@ replace_spin_keys (pthread_spinlock_t *old, pthread_spinlock_t new)
 
   if (EPERM == pthread_spin_destroy (old))
     {
-#define THREADERR "Error cleaning up spin_keys for thread "
-#define THREADERR_LEN ((sizeof (THREADERR) / sizeof (*THREADERR)) - 1)
-#define THREADID_LEN THREADERR_LEN + 66 + 1 + 1
-      int i;
-      char thread_id[THREADID_LEN] = THREADERR;
-      _ultoa ((unsigned long) GetCurrentThreadId (), &thread_id[THREADERR_LEN], 10);
-      for (i = THREADERR_LEN; thread_id[i] != '\0' && i < THREADID_LEN - 1; i++)
-        {
-        }
-      if (i < THREADID_LEN - 1)
-        {
-          thread_id[i] = '\n';
-          thread_id[i + 1] = '\0';
-        }
+#define THREADERR "Error cleaning up spin_keys for thread %lu.\n"
+      char threaderr[sizeof(THREADERR) + 8] = { 0 };
+      snprintf(threaderr, sizeof(threaderr), THREADERR, GetCurrentThreadId());
 #undef THREADERR
-#undef THREADERR_LEN
-#undef THREADID_LEN
-      OutputDebugStringA (thread_id);
+      OutputDebugStringA (threaderr);
       abort ();
     }
 
@@ -423,7 +424,7 @@ replace_spin_keys (pthread_spinlock_t *old, pthread_spinlock_t new)
 }
 
 /* Hook for TLS-based deregistration/registration of thread.  */
-static BOOL WINAPI
+static void WINAPI
 __dyn_tls_pthread (HANDLE hDllHandle, DWORD dwReason, LPVOID lpreserved)
 {
   _pthread_v *t = NULL;
@@ -434,7 +435,8 @@ __dyn_tls_pthread (HANDLE hDllHandle, DWORD dwReason, LPVOID lpreserved)
 #if defined(USE_VEH_FOR_MSC_SETTHREADNAME)
       if (lpreserved == NULL && SetThreadName_VEH_handle != NULL)
         {
-          RemoveVectoredExceptionHandler (SetThreadName_VEH_handle);
+          if (RemoveVectoredExceptionHandlerFuncPtr != NULL)
+            RemoveVectoredExceptionHandlerFuncPtr (SetThreadName_VEH_handle);
           SetThreadName_VEH_handle = NULL;
         }
 #endif
@@ -443,7 +445,10 @@ __dyn_tls_pthread (HANDLE hDllHandle, DWORD dwReason, LPVOID lpreserved)
   else if (dwReason == DLL_PROCESS_ATTACH)
     {
 #if defined(USE_VEH_FOR_MSC_SETTHREADNAME)
-      SetThreadName_VEH_handle = AddVectoredExceptionHandler (1, &SetThreadName_VEH);
+      if (AddVectoredExceptionHandlerFuncPtr != NULL)
+        SetThreadName_VEH_handle = AddVectoredExceptionHandlerFuncPtr (1, &SetThreadName_VEH);
+      else
+        SetThreadName_VEH_handle = NULL;
       /* Can't do anything on error anyway, check for NULL later */
 #endif
     }
@@ -486,7 +491,7 @@ __dyn_tls_pthread (HANDLE hDllHandle, DWORD dwReason, LPVOID lpreserved)
 	      push_pthread_mem (t);
 	      t = NULL;
 	      TlsSetValue (_pthread_tls, t);
-	      return TRUE;
+	      return;
 	    }
 	  pthread_mutex_destroy(&t->p_clock);
 	  replace_spin_keys (&t->spin_keys, new_spin_keys);
@@ -500,18 +505,39 @@ __dyn_tls_pthread (HANDLE hDllHandle, DWORD dwReason, LPVOID lpreserved)
 	  replace_spin_keys (&t->spin_keys, new_spin_keys);
 	}
     }
-  return TRUE;
 }
 
 /* TLS-runtime section variable.  */
-#ifdef _MSC_VER
-#pragma section(".CRT$XLF", shared)
-#endif
-PIMAGE_TLS_CALLBACK WINPTHREADS_ATTRIBUTE((WINPTHREADS_SECTION(".CRT$XLF"))) __xl_f  = (PIMAGE_TLS_CALLBACK) __dyn_tls_pthread;
-#ifdef _MSC_VER
-#pragma data_seg()
+
+#if defined(_MSC_VER)
+/* Force a reference to _tls_used to make the linker create the TLS
+ * directory if it's not already there.  (e.g. if __declspec(thread)
+ * is not used).
+ * Force a reference to __xl_f to prevent whole program optimization
+ * from discarding the variable. */
+
+/* On x86, symbols are prefixed with an underscore. */
+# if defined(_M_IX86)
+#   pragma comment(linker, "/include:__tls_used")
+#   pragma comment(linker, "/include:___xl_f")
+# else
+#   pragma comment(linker, "/include:_tls_used")
+#   pragma comment(linker, "/include:__xl_f")
+# endif
+
+/* .CRT$XLA to .CRT$XLZ is an array of PIMAGE_TLS_CALLBACK
+ * pointers. Pick an arbitrary location for our callback.
+ *
+ * See VC\...\crt\src\vcruntime\tlssup.cpp for reference. */
+
+# pragma section(".CRT$XLF", long, read)
 #endif
 
+WINPTHREADS_ATTRIBUTE((WINPTHREADS_SECTION(".CRT$XLF")))
+extern const PIMAGE_TLS_CALLBACK __xl_f;
+const PIMAGE_TLS_CALLBACK __xl_f = __dyn_tls_pthread;
+
+
 #ifdef WINPTHREAD_DBG
 static int print_state = 0;
 void thread_print_set (int state)
@@ -525,13 +551,13 @@ thread_print (volatile pthread_t t, char *txt)
     if (!print_state)
       return;
     if (!t)
-      printf("T%p %d %s\n",NULL,(int)GetCurrentThreadId(),txt);
+      printf("T%p %lu %s\n",NULL,GetCurrentThreadId(),txt);
     else
       {
-	printf("T%p %d V=%0X H=%p %s\n",
-	    __pth_gpointer_locked (t), 
-	    (int)GetCurrentThreadId(), 
-	    (int) (__pth_gpointer_locked (t))->valid, 
+	printf("T%p %lu V=%0X H=%p %s\n",
+	    (void *) __pth_gpointer_locked (t),
+	    GetCurrentThreadId(),
+	    (__pth_gpointer_locked (t))->valid,
 	    (__pth_gpointer_locked (t))->h,
 	    txt
 	    );
@@ -603,7 +629,7 @@ leaveOnceObject (collect_once_t *c)
 	}
     }
   else
-    fprintf(stderr, "%p not found?!?!\n", c);
+    fprintf(stderr, "%p not found?!?!\n", (void *) c);
   pthread_spin_unlock (&once_global);
 }
 
@@ -634,7 +660,7 @@ _pthread_once_raw (pthread_once_t *o, void (*func)(void))
       *o = 1;
     }
   else if (*o != 1)
-    fprintf (stderr," once %p is %d\n", o, (int) *o);
+    fprintf (stderr," once %p is %ld\n", (void *) o, (long) *o);
   pthread_mutex_unlock(&co->m);
   leaveOnceObject(co);
 
@@ -759,7 +785,7 @@ pthread_once (pthread_once_t *o, void (*func)(void))
       *o = 1;
     }
   else if (*o != 1)
-    fprintf (stderr," once %p is %d\n", o, (int) *o);
+    fprintf (stderr," once %p is %ld\n", (void *) o, (long) *o);
   pthread_mutex_unlock(&co->m);
   leaveOnceObject(co);
 
@@ -1499,7 +1525,7 @@ void _fpreset (void);
 __attribute__((force_align_arg_pointer))
 #  endif
 #endif
-int
+unsigned __stdcall
 pthread_create_wrapper (void *args)
 {
   unsigned rslt = 0;
@@ -1520,19 +1546,23 @@ pthread_create_wrapper (void *args)
       intptr_t trslt = (intptr_t) 128;
       /* Provide to this thread a default exception handler.  */
       #ifdef __SEH__
-	asm ("\t.tl_start:\n"
-	  "\t.seh_handler __C_specific_handler, @except\n"
-	  "\t.seh_handlerdata\n"
-	  "\t.long 1\n"
-	  "\t.rva .tl_start, .tl_end, _gnu_exception_handler ,.tl_end\n"
-	  "\t.text"
-	  );
+	asm ("\t.tl_start:\n");
       #endif      /* Call function and save return value */
       pthread_mutex_unlock (&mtx_pthr_locked);
       if (tv->func)
         trslt = (intptr_t) tv->func(tv->ret_arg);
       #ifdef __SEH__
-	asm ("\tnop\n\t.tl_end: nop\n");
+	asm ("\tnop\n\t.tl_end: nop\n"
+#ifdef __arm__
+	  "\t.seh_handler __C_specific_handler, %except\n"
+#else
+	  "\t.seh_handler __C_specific_handler, @except\n"
+#endif
+	  "\t.seh_handlerdata\n"
+	  "\t.long 1\n"
+	  "\t.rva .tl_start, .tl_end, _gnu_exception_handler ,.tl_end\n"
+	  "\t.text"
+	  );
       #endif
       pthread_mutex_lock (&mtx_pthr_locked);
       tv->ret_arg = (void*) trslt;
@@ -1578,9 +1608,12 @@ pthread_create (pthread_t *th, const pthread_attr_t *attr, void *(* func)(void *
   HANDLE thrd = NULL;
   int redo = 0;
   struct _pthread_v *tv;
-  size_t ssize = 0;
+  unsigned int ssize = 0;
   pthread_spinlock_t new_spin_keys = PTHREAD_SPINLOCK_INITIALIZER;
 
+  if (attr && attr->s_size > UINT_MAX)
+    return EINVAL;
+
   if ((tv = pop_pthread_mem ()) == NULL)
     return EAGAIN;
 
@@ -1620,7 +1653,7 @@ pthread_create (pthread_t *th, const pthread_attr_t *attr, void *(* func)(void *
     {
       int inh = 0;
       tv->p_state = attr->p_state;
-      ssize = attr->s_size;
+      ssize = (unsigned int)attr->s_size;
       pthread_attr_getinheritsched (attr, &inh);
       if (inh)
 	{
@@ -1633,7 +1666,7 @@ pthread_create (pthread_t *th, const pthread_attr_t *attr, void *(* func)(void *
   /* Make sure tv->h has value of INVALID_HANDLE_VALUE */
   _ReadWriteBarrier();
 
-  thrd = (HANDLE) _beginthreadex(NULL, ssize, (unsigned int (__stdcall *)(void *))pthread_create_wrapper, tv, 0x4/*CREATE_SUSPEND*/, NULL);
+  thrd = (HANDLE) _beginthreadex(NULL, ssize, pthread_create_wrapper, tv, 0x4/*CREATE_SUSPEND*/, NULL);
   if (thrd == INVALID_HANDLE_VALUE)
     thrd = 0;
   /* Failed */
